// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package apis

import (
    "github.com/jdcloud-api/jdcloud-sdk-go/core"
)

type CreateAppRequest struct {

    core.JDCloudRequest

    /* 地域编码，参考OpenAPI公共说明  */
    RegionId string `json:"regionId"`

    /* 应用名 (Optional) */
    ClientName *string `json:"clientName"`

    /* 客户端认证方式<br> - none：不设置客户端密码（不推荐）<br> - client_secret_post：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的body<br> - client_secret_basic：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的header<br> 支持以下值：<br> （1）none<br> （2）client_secret_post<br> （3）client_secret_basic (Optional) */
    TokenEndpointAuthMethod *string `json:"tokenEndpointAuthMethod"`

    /* 支持的OAuth类型：<br> - authorization_code：OAuth2授权码模式<br> - implicit：OAuth2隐式授权模式<br> - refresh_token：启用刷新令牌 支持以下值：<br> （1）authorization_code<br> （2）authorization_code,refresh_token<br> （3）authorization_code,implicit<br> （4）authorization_code,implicit,refresh_token<br> （5）implicit<br> 注：如果grantTypes指定了refresh_token，应用将可以使用刷新令牌；如果在创建应用时未指定，则应用不能使用刷新令牌；任何时候应用都可以调用“更新应用”接口更改grantTypes设置 (Optional) */
    GrantTypes *string `json:"grantTypes"`

    /* 回调地址，最多4个，多个url之间用逗号,分隔，每个url长度不超过1000，url不支持#符号 (Optional) */
    RedirectUris *string `json:"redirectUris"`

    /* 应用介绍地址，url不支持#符号 (Optional) */
    ClientUri *string `json:"clientUri"`

    /* 应用logo地址，url不支持#符号 (Optional) */
    LogoUri *string `json:"logoUri"`

    /* 应用服务协议地址，url不支持#符号 (Optional) */
    TosUri *string `json:"tosUri"`

    /* 应用隐私政策地址，url不支持#符号 (Optional) */
    PolicyUri *string `json:"policyUri"`

    /* OAuth scope范围，支持的值为：<br/> （1）openid：用OpenID Connect协议进行身份认证<br/> 指定scope为openid，并在Authorization Endpoint请求该scope，京东云将返回用户的OpenID令牌；如果在创建应用时未指明该值，则应用不能请求OpenID令牌；任何时候应用都可以调用“更新应用”更改该设置 (Optional) */
    Scope *string `json:"scope"`

    /* JWKS地址，url不支持#符号<br/>jwksUri和jwks传一个即可 (Optional) */
    JwksUri *string `json:"jwksUri"`

    /* JWKS (Optional) */
    Jwks *string `json:"jwks"`

    /* 应用联系信息 (Optional) */
    Contacts *string `json:"contacts"`

    /* 应用扩展信息 (Optional) */
    Extension *string `json:"extension"`

    /* 访问令牌有效期，值的范围为 600 秒到 6x3600=21,600 秒，即10分钟-6小时 (Optional) */
    AccessTokenValiditySeconds *int `json:"accessTokenValiditySeconds"`

    /* 刷新令牌有效期，值的范围为 30x24x3600=2,592,000 秒到 365x24x3600=31,536,000 秒，即30天-365天<br/><br/> 注：当 GrantTypes 包含 refresh_token 时，refreshTokenValiditySeconds 为必传参数 (Optional) */
    RefreshTokenValiditySeconds *int `json:"refreshTokenValiditySeconds"`

    /* 是否为多租户应用<br/> "false"：该应用仅支持当前创建应用的租户访问，其他京东云租户无法访问<br/>        "true"：该应用支持其他京东云租户访问，但当前创建应用的租户不能访问 (Optional) */
    MultiTenant *bool `json:"multiTenant"`

    /* 应用的密码，支持8-255位长度的ASCII可打印字符，建议使用足够复杂的密码策略<br/><br/>        注：当TokenEndpointAuthMethod不等于none时，secret为必传参数；反之，当指定了secret时，TokenEndpointAuthMethod不能等于none<br/>京东云将不可逆加密secret，因此您无法再次从京东云查看该密码，但您可以随时通过更新应用重新设置secret (Optional) */
    Secret *string `json:"secret"`

    /* 能访问应用的账号类型，支持以下值：<br/> （1）root：支持主账号访问，子用户无法访问<br/> （2）sub：子用户账号，使用主账号不能访问<br/><br/> 注：multiTenant和userType的组合指定了应用的用户人群，典型的应用场景如：<br/> （1）应用向当前租户下的子用户开放（2）应用向京东云其他租户主账号开放 (Optional) */
    UserType *string `json:"userType"`
}

/*
 * param regionId: 地域编码，参考OpenAPI公共说明 (Required)
 *
 * @Deprecated, not compatible when mandatory parameters changed
 */
func NewCreateAppRequest(
    regionId string,
) *CreateAppRequest {

	return &CreateAppRequest{
        JDCloudRequest: core.JDCloudRequest{
			URL:     "/regions/{regionId}/app",
			Method:  "POST",
			Header:  nil,
			Version: "v1",
		},
        RegionId: regionId,
	}
}

/*
 * param regionId: 地域编码，参考OpenAPI公共说明 (Required)
 * param clientName: 应用名 (Optional)
 * param tokenEndpointAuthMethod: 客户端认证方式<br> - none：不设置客户端密码（不推荐）<br> - client_secret_post：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的body<br> - client_secret_basic：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的header<br> 支持以下值：<br> （1）none<br> （2）client_secret_post<br> （3）client_secret_basic (Optional)
 * param grantTypes: 支持的OAuth类型：<br> - authorization_code：OAuth2授权码模式<br> - implicit：OAuth2隐式授权模式<br> - refresh_token：启用刷新令牌 支持以下值：<br> （1）authorization_code<br> （2）authorization_code,refresh_token<br> （3）authorization_code,implicit<br> （4）authorization_code,implicit,refresh_token<br> （5）implicit<br> 注：如果grantTypes指定了refresh_token，应用将可以使用刷新令牌；如果在创建应用时未指定，则应用不能使用刷新令牌；任何时候应用都可以调用“更新应用”接口更改grantTypes设置 (Optional)
 * param redirectUris: 回调地址，最多4个，多个url之间用逗号,分隔，每个url长度不超过1000，url不支持#符号 (Optional)
 * param clientUri: 应用介绍地址，url不支持#符号 (Optional)
 * param logoUri: 应用logo地址，url不支持#符号 (Optional)
 * param tosUri: 应用服务协议地址，url不支持#符号 (Optional)
 * param policyUri: 应用隐私政策地址，url不支持#符号 (Optional)
 * param scope: OAuth scope范围，支持的值为：<br/> （1）openid：用OpenID Connect协议进行身份认证<br/> 指定scope为openid，并在Authorization Endpoint请求该scope，京东云将返回用户的OpenID令牌；如果在创建应用时未指明该值，则应用不能请求OpenID令牌；任何时候应用都可以调用“更新应用”更改该设置 (Optional)
 * param jwksUri: JWKS地址，url不支持#符号<br/>jwksUri和jwks传一个即可 (Optional)
 * param jwks: JWKS (Optional)
 * param contacts: 应用联系信息 (Optional)
 * param extension: 应用扩展信息 (Optional)
 * param accessTokenValiditySeconds: 访问令牌有效期，值的范围为 600 秒到 6x3600=21,600 秒，即10分钟-6小时 (Optional)
 * param refreshTokenValiditySeconds: 刷新令牌有效期，值的范围为 30x24x3600=2,592,000 秒到 365x24x3600=31,536,000 秒，即30天-365天<br/><br/> 注：当 GrantTypes 包含 refresh_token 时，refreshTokenValiditySeconds 为必传参数 (Optional)
 * param multiTenant: 是否为多租户应用<br/> "false"：该应用仅支持当前创建应用的租户访问，其他京东云租户无法访问<br/>        "true"：该应用支持其他京东云租户访问，但当前创建应用的租户不能访问 (Optional)
 * param secret: 应用的密码，支持8-255位长度的ASCII可打印字符，建议使用足够复杂的密码策略<br/><br/>        注：当TokenEndpointAuthMethod不等于none时，secret为必传参数；反之，当指定了secret时，TokenEndpointAuthMethod不能等于none<br/>京东云将不可逆加密secret，因此您无法再次从京东云查看该密码，但您可以随时通过更新应用重新设置secret (Optional)
 * param userType: 能访问应用的账号类型，支持以下值：<br/> （1）root：支持主账号访问，子用户无法访问<br/> （2）sub：子用户账号，使用主账号不能访问<br/><br/> 注：multiTenant和userType的组合指定了应用的用户人群，典型的应用场景如：<br/> （1）应用向当前租户下的子用户开放（2）应用向京东云其他租户主账号开放 (Optional)
 */
func NewCreateAppRequestWithAllParams(
    regionId string,
    clientName *string,
    tokenEndpointAuthMethod *string,
    grantTypes *string,
    redirectUris *string,
    clientUri *string,
    logoUri *string,
    tosUri *string,
    policyUri *string,
    scope *string,
    jwksUri *string,
    jwks *string,
    contacts *string,
    extension *string,
    accessTokenValiditySeconds *int,
    refreshTokenValiditySeconds *int,
    multiTenant *bool,
    secret *string,
    userType *string,
) *CreateAppRequest {

    return &CreateAppRequest{
        JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/app",
            Method:  "POST",
            Header:  nil,
            Version: "v1",
        },
        RegionId: regionId,
        ClientName: clientName,
        TokenEndpointAuthMethod: tokenEndpointAuthMethod,
        GrantTypes: grantTypes,
        RedirectUris: redirectUris,
        ClientUri: clientUri,
        LogoUri: logoUri,
        TosUri: tosUri,
        PolicyUri: policyUri,
        Scope: scope,
        JwksUri: jwksUri,
        Jwks: jwks,
        Contacts: contacts,
        Extension: extension,
        AccessTokenValiditySeconds: accessTokenValiditySeconds,
        RefreshTokenValiditySeconds: refreshTokenValiditySeconds,
        MultiTenant: multiTenant,
        Secret: secret,
        UserType: userType,
    }
}

/* This constructor has better compatible ability when API parameters changed */
func NewCreateAppRequestWithoutParam() *CreateAppRequest {

    return &CreateAppRequest{
            JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/app",
            Method:  "POST",
            Header:  nil,
            Version: "v1",
        },
    }
}

/* param regionId: 地域编码，参考OpenAPI公共说明(Required) */
func (r *CreateAppRequest) SetRegionId(regionId string) {
    r.RegionId = regionId
}

/* param clientName: 应用名(Optional) */
func (r *CreateAppRequest) SetClientName(clientName string) {
    r.ClientName = &clientName
}

/* param tokenEndpointAuthMethod: 客户端认证方式<br> - none：不设置客户端密码（不推荐）<br> - client_secret_post：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的body<br> - client_secret_basic：客户端必须设置密码，且该密码需要在OAuth2 Token Endpoint提供于请求的header<br> 支持以下值：<br> （1）none<br> （2）client_secret_post<br> （3）client_secret_basic(Optional) */
func (r *CreateAppRequest) SetTokenEndpointAuthMethod(tokenEndpointAuthMethod string) {
    r.TokenEndpointAuthMethod = &tokenEndpointAuthMethod
}

/* param grantTypes: 支持的OAuth类型：<br> - authorization_code：OAuth2授权码模式<br> - implicit：OAuth2隐式授权模式<br> - refresh_token：启用刷新令牌 支持以下值：<br> （1）authorization_code<br> （2）authorization_code,refresh_token<br> （3）authorization_code,implicit<br> （4）authorization_code,implicit,refresh_token<br> （5）implicit<br> 注：如果grantTypes指定了refresh_token，应用将可以使用刷新令牌；如果在创建应用时未指定，则应用不能使用刷新令牌；任何时候应用都可以调用“更新应用”接口更改grantTypes设置(Optional) */
func (r *CreateAppRequest) SetGrantTypes(grantTypes string) {
    r.GrantTypes = &grantTypes
}

/* param redirectUris: 回调地址，最多4个，多个url之间用逗号,分隔，每个url长度不超过1000，url不支持#符号(Optional) */
func (r *CreateAppRequest) SetRedirectUris(redirectUris string) {
    r.RedirectUris = &redirectUris
}

/* param clientUri: 应用介绍地址，url不支持#符号(Optional) */
func (r *CreateAppRequest) SetClientUri(clientUri string) {
    r.ClientUri = &clientUri
}

/* param logoUri: 应用logo地址，url不支持#符号(Optional) */
func (r *CreateAppRequest) SetLogoUri(logoUri string) {
    r.LogoUri = &logoUri
}

/* param tosUri: 应用服务协议地址，url不支持#符号(Optional) */
func (r *CreateAppRequest) SetTosUri(tosUri string) {
    r.TosUri = &tosUri
}

/* param policyUri: 应用隐私政策地址，url不支持#符号(Optional) */
func (r *CreateAppRequest) SetPolicyUri(policyUri string) {
    r.PolicyUri = &policyUri
}

/* param scope: OAuth scope范围，支持的值为：<br/> （1）openid：用OpenID Connect协议进行身份认证<br/> 指定scope为openid，并在Authorization Endpoint请求该scope，京东云将返回用户的OpenID令牌；如果在创建应用时未指明该值，则应用不能请求OpenID令牌；任何时候应用都可以调用“更新应用”更改该设置(Optional) */
func (r *CreateAppRequest) SetScope(scope string) {
    r.Scope = &scope
}

/* param jwksUri: JWKS地址，url不支持#符号<br/>jwksUri和jwks传一个即可(Optional) */
func (r *CreateAppRequest) SetJwksUri(jwksUri string) {
    r.JwksUri = &jwksUri
}

/* param jwks: JWKS(Optional) */
func (r *CreateAppRequest) SetJwks(jwks string) {
    r.Jwks = &jwks
}

/* param contacts: 应用联系信息(Optional) */
func (r *CreateAppRequest) SetContacts(contacts string) {
    r.Contacts = &contacts
}

/* param extension: 应用扩展信息(Optional) */
func (r *CreateAppRequest) SetExtension(extension string) {
    r.Extension = &extension
}

/* param accessTokenValiditySeconds: 访问令牌有效期，值的范围为 600 秒到 6x3600=21,600 秒，即10分钟-6小时(Optional) */
func (r *CreateAppRequest) SetAccessTokenValiditySeconds(accessTokenValiditySeconds int) {
    r.AccessTokenValiditySeconds = &accessTokenValiditySeconds
}

/* param refreshTokenValiditySeconds: 刷新令牌有效期，值的范围为 30x24x3600=2,592,000 秒到 365x24x3600=31,536,000 秒，即30天-365天<br/><br/> 注：当 GrantTypes 包含 refresh_token 时，refreshTokenValiditySeconds 为必传参数(Optional) */
func (r *CreateAppRequest) SetRefreshTokenValiditySeconds(refreshTokenValiditySeconds int) {
    r.RefreshTokenValiditySeconds = &refreshTokenValiditySeconds
}

/* param multiTenant: 是否为多租户应用<br/> "false"：该应用仅支持当前创建应用的租户访问，其他京东云租户无法访问<br/>        "true"：该应用支持其他京东云租户访问，但当前创建应用的租户不能访问(Optional) */
func (r *CreateAppRequest) SetMultiTenant(multiTenant bool) {
    r.MultiTenant = &multiTenant
}

/* param secret: 应用的密码，支持8-255位长度的ASCII可打印字符，建议使用足够复杂的密码策略<br/><br/>        注：当TokenEndpointAuthMethod不等于none时，secret为必传参数；反之，当指定了secret时，TokenEndpointAuthMethod不能等于none<br/>京东云将不可逆加密secret，因此您无法再次从京东云查看该密码，但您可以随时通过更新应用重新设置secret(Optional) */
func (r *CreateAppRequest) SetSecret(secret string) {
    r.Secret = &secret
}

/* param userType: 能访问应用的账号类型，支持以下值：<br/> （1）root：支持主账号访问，子用户无法访问<br/> （2）sub：子用户账号，使用主账号不能访问<br/><br/> 注：multiTenant和userType的组合指定了应用的用户人群，典型的应用场景如：<br/> （1）应用向当前租户下的子用户开放（2）应用向京东云其他租户主账号开放(Optional) */
func (r *CreateAppRequest) SetUserType(userType string) {
    r.UserType = &userType
}

// GetRegionId returns path parameter 'regionId' if exist,
// otherwise return empty string
func (r CreateAppRequest) GetRegionId() string {
    return r.RegionId
}

type CreateAppResponse struct {
    RequestID string `json:"requestId"`
    Error core.ErrorResponse `json:"error"`
    Result CreateAppResult `json:"result"`
}

type CreateAppResult struct {
    ClientId string `json:"clientId"`
    ClientName string `json:"clientName"`
    TokenEndpointAuthMethod string `json:"tokenEndpointAuthMethod"`
    GrantTypes string `json:"grantTypes"`
    ResponseTypes string `json:"responseTypes"`
    RedirectUris string `json:"redirectUris"`
    ClientUri string `json:"clientUri"`
    LogoUri string `json:"logoUri"`
    TosUri string `json:"tosUri"`
    PolicyUri string `json:"policyUri"`
    Scope string `json:"scope"`
    JwksUri string `json:"jwksUri"`
    Jwks string `json:"jwks"`
    Contacts string `json:"contacts"`
    Extension string `json:"extension"`
    AccessTokenValiditySeconds int `json:"accessTokenValiditySeconds"`
    RefreshTokenValiditySeconds int `json:"refreshTokenValiditySeconds"`
    MultiTenant bool `json:"multiTenant"`
    SecretUpdateTime int64 `json:"secretUpdateTime"`
    UpdateTime int64 `json:"updateTime"`
    CreateTime int64 `json:"createTime"`
    Account string `json:"account"`
    UserType string `json:"userType"`
}