// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package apis

import (
    "github.com/jdcloud-api/jdcloud-sdk-go/core"
    kubernetes "github.com/jdcloud-api/jdcloud-sdk-go/services/kubernetes/models"
)

type CreateClusterRequest struct {

    core.JDCloudRequest

    /* 地域 ID  */
    RegionId string `json:"regionId"`

    /* 名称（同一用户的 cluster 允许重名）  */
    Name string `json:"name"`

    /* 描述 (Optional) */
    Description *string `json:"description"`

    /* 默认开启 basicAuth与clientCertificate最少选择一个 (Optional) */
    BasicAuth *bool `json:"basicAuth"`

    /* 默认开启 clientCertificate (Optional) */
    ClientCertificate *bool `json:"clientCertificate"`

    /* kubernetes的版本 (Optional) */
    Version *string `json:"version"`

    /* 集群所在的az  */
    Azs []string `json:"azs"`

    /* pod 创建参数  */
    NodeGroup *kubernetes.NodeGroupSpec `json:"nodeGroup"`

    /* k8s的master的cidr  */
    MasterCidr string `json:"masterCidr"`

    /* 用户的AccessKey，插件调用open-api时的认证凭证  */
    AccessKey string `json:"accessKey"`

    /* 用户的SecretKey，插件调用open-api时的认证凭证  */
    SecretKey string `json:"secretKey"`

    /* 是否启用用户自定义监控，默认不启用 (Optional) */
    UserMetrics *bool `json:"userMetrics"`
}

/*
 * param regionId: 地域 ID (Required)
 * param name: 名称（同一用户的 cluster 允许重名） (Required)
 * param azs: 集群所在的az (Required)
 * param nodeGroup: pod 创建参数 (Required)
 * param masterCidr: k8s的master的cidr (Required)
 * param accessKey: 用户的AccessKey，插件调用open-api时的认证凭证 (Required)
 * param secretKey: 用户的SecretKey，插件调用open-api时的认证凭证 (Required)
 *
 * @Deprecated, not compatible when mandatory parameters changed
 */
func NewCreateClusterRequest(
    regionId string,
    name string,
    azs []string,
    nodeGroup *kubernetes.NodeGroupSpec,
    masterCidr string,
    accessKey string,
    secretKey string,
) *CreateClusterRequest {

	return &CreateClusterRequest{
        JDCloudRequest: core.JDCloudRequest{
			URL:     "/regions/{regionId}/clusters",
			Method:  "POST",
			Header:  nil,
			Version: "v1",
		},
        RegionId: regionId,
        Name: name,
        Azs: azs,
        NodeGroup: nodeGroup,
        MasterCidr: masterCidr,
        AccessKey: accessKey,
        SecretKey: secretKey,
	}
}

/*
 * param regionId: 地域 ID (Required)
 * param name: 名称（同一用户的 cluster 允许重名） (Required)
 * param description: 描述 (Optional)
 * param basicAuth: 默认开启 basicAuth与clientCertificate最少选择一个 (Optional)
 * param clientCertificate: 默认开启 clientCertificate (Optional)
 * param version: kubernetes的版本 (Optional)
 * param azs: 集群所在的az (Required)
 * param nodeGroup: pod 创建参数 (Required)
 * param masterCidr: k8s的master的cidr (Required)
 * param accessKey: 用户的AccessKey，插件调用open-api时的认证凭证 (Required)
 * param secretKey: 用户的SecretKey，插件调用open-api时的认证凭证 (Required)
 * param userMetrics: 是否启用用户自定义监控，默认不启用 (Optional)
 */
func NewCreateClusterRequestWithAllParams(
    regionId string,
    name string,
    description *string,
    basicAuth *bool,
    clientCertificate *bool,
    version *string,
    azs []string,
    nodeGroup *kubernetes.NodeGroupSpec,
    masterCidr string,
    accessKey string,
    secretKey string,
    userMetrics *bool,
) *CreateClusterRequest {

    return &CreateClusterRequest{
        JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/clusters",
            Method:  "POST",
            Header:  nil,
            Version: "v1",
        },
        RegionId: regionId,
        Name: name,
        Description: description,
        BasicAuth: basicAuth,
        ClientCertificate: clientCertificate,
        Version: version,
        Azs: azs,
        NodeGroup: nodeGroup,
        MasterCidr: masterCidr,
        AccessKey: accessKey,
        SecretKey: secretKey,
        UserMetrics: userMetrics,
    }
}

/* This constructor has better compatible ability when API parameters changed */
func NewCreateClusterRequestWithoutParam() *CreateClusterRequest {

    return &CreateClusterRequest{
            JDCloudRequest: core.JDCloudRequest{
            URL:     "/regions/{regionId}/clusters",
            Method:  "POST",
            Header:  nil,
            Version: "v1",
        },
    }
}

/* param regionId: 地域 ID(Required) */
func (r *CreateClusterRequest) SetRegionId(regionId string) {
    r.RegionId = regionId
}

/* param name: 名称（同一用户的 cluster 允许重名）(Required) */
func (r *CreateClusterRequest) SetName(name string) {
    r.Name = name
}

/* param description: 描述(Optional) */
func (r *CreateClusterRequest) SetDescription(description string) {
    r.Description = &description
}

/* param basicAuth: 默认开启 basicAuth与clientCertificate最少选择一个(Optional) */
func (r *CreateClusterRequest) SetBasicAuth(basicAuth bool) {
    r.BasicAuth = &basicAuth
}

/* param clientCertificate: 默认开启 clientCertificate(Optional) */
func (r *CreateClusterRequest) SetClientCertificate(clientCertificate bool) {
    r.ClientCertificate = &clientCertificate
}

/* param version: kubernetes的版本(Optional) */
func (r *CreateClusterRequest) SetVersion(version string) {
    r.Version = &version
}

/* param azs: 集群所在的az(Required) */
func (r *CreateClusterRequest) SetAzs(azs []string) {
    r.Azs = azs
}

/* param nodeGroup: pod 创建参数(Required) */
func (r *CreateClusterRequest) SetNodeGroup(nodeGroup *kubernetes.NodeGroupSpec) {
    r.NodeGroup = nodeGroup
}

/* param masterCidr: k8s的master的cidr(Required) */
func (r *CreateClusterRequest) SetMasterCidr(masterCidr string) {
    r.MasterCidr = masterCidr
}

/* param accessKey: 用户的AccessKey，插件调用open-api时的认证凭证(Required) */
func (r *CreateClusterRequest) SetAccessKey(accessKey string) {
    r.AccessKey = accessKey
}

/* param secretKey: 用户的SecretKey，插件调用open-api时的认证凭证(Required) */
func (r *CreateClusterRequest) SetSecretKey(secretKey string) {
    r.SecretKey = secretKey
}

/* param userMetrics: 是否启用用户自定义监控，默认不启用(Optional) */
func (r *CreateClusterRequest) SetUserMetrics(userMetrics bool) {
    r.UserMetrics = &userMetrics
}

// GetRegionId returns path parameter 'regionId' if exist,
// otherwise return empty string
func (r CreateClusterRequest) GetRegionId() string {
    return r.RegionId
}

type CreateClusterResponse struct {
    RequestID string `json:"requestId"`
    Error core.ErrorResponse `json:"error"`
    Result CreateClusterResult `json:"result"`
}

type CreateClusterResult struct {
    ClusterId string `json:"clusterId"`
}