// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package client

import (
    "github.com/jdcloud-api/jdcloud-sdk-go/core"
    mps "github.com/jdcloud-api/jdcloud-sdk-go/services/mps/apis"
    "encoding/json"
    "errors"
)

type MpsClient struct {
    core.JDCloudClient
}

func NewMpsClient(credential *core.Credential) *MpsClient {
    if credential == nil {
        return nil
    }

    config := core.NewConfig()
    config.SetEndpoint("mps.jdcloud-api.com")

    return &MpsClient{
        core.JDCloudClient{
            Credential:  *credential,
            Config:      *config,
            ServiceName: "mps",
            Revision:    "0.4.1",
            Logger:      core.NewDefaultLogger(core.LogInfo),
        }}
}

func (c *MpsClient) SetConfig(config *core.Config) {
    c.Config = *config
}

func (c *MpsClient) SetLogger(logger core.Logger) {
    c.Logger = logger
}

/* 创建截图任务，创建成功时返回任务ID。本接口用于截取指定时间点的画面。 */
func (c *MpsClient) CreateThumbnailTask(request *mps.CreateThumbnailTaskRequest) (*mps.CreateThumbnailTaskResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.CreateThumbnailTaskResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 查询截图任务，返回满足查询条件的任务列表。 */
func (c *MpsClient) ListThumbnailTask(request *mps.ListThumbnailTaskRequest) (*mps.ListThumbnailTaskResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.ListThumbnailTaskResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 删除bucket的图片样式分隔符配置 */
func (c *MpsClient) DeleteStyleDelimiter(request *mps.DeleteStyleDelimiterRequest) (*mps.DeleteStyleDelimiterResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.DeleteStyleDelimiterResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 获取媒体处理通知 */
func (c *MpsClient) GetNotification(request *mps.GetNotificationRequest) (*mps.GetNotificationResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.GetNotificationResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 根据任务ID获取截图任务。 */
func (c *MpsClient) GetThumbnailTask(request *mps.GetThumbnailTaskRequest) (*mps.GetThumbnailTaskResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.GetThumbnailTaskResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 设置媒体处理通知, 在设置Notification时会对endpoint进行校验, 设置时会对endpoint发一条SubscriptionConfirmation(x-jdcloud-message-type头)的通知, 要求把Message内容进行base64编码返回给系统(body)进行校验 */
func (c *MpsClient) SetNotification(request *mps.SetNotificationRequest) (*mps.SetNotificationResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.SetNotificationResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 设置图片样式分隔符 */
func (c *MpsClient) SetStyleDelimiter(request *mps.SetStyleDelimiterRequest) (*mps.SetStyleDelimiterResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.SetStyleDelimiterResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

/* 获取bucket的图片样式分隔符配置 */
func (c *MpsClient) GetStyleDelimiter(request *mps.GetStyleDelimiterRequest) (*mps.GetStyleDelimiterResponse, error) {
    if request == nil {
        return nil, errors.New("Request object is nil. ")
    }
    resp, err := c.Send(request, c.ServiceName)
    if err != nil {
        return nil, err
    }

    jdResp := &mps.GetStyleDelimiterResponse{}
    err = json.Unmarshal(resp, jdResp)
    if err != nil {
        c.Logger.Log(core.LogError, "Unmarshal json failed, resp: %s", string(resp))
        return nil, err
    }

    return jdResp, err
}

